%This case study applies percentile regression to the return-based style
%classification of a mutual fund.  The procedure regresses fund return by
%several indices as explanatory variables. The estimated coefficients
%represent the funds style with respect to each of the indices.
%This case study regresses the S&P 500 index and Fidelity Magellan Fund on
%the Russell Value Index (RUJ), RUSSELL 1000 VALUE INDEX (RLV), 
%Russell 2000 Growth Index (RUO) and Russell 1000 Growth Index (RLG). 
%The confidence level in quantile regression varies from 0.1 to 0.9. 

clc
clear;

thistime = cputime;
warning('off', 'all');

% load packed data
load Style_Classification_Quantile_Regression_PM_data_Toolbox.mat %toolboxstruc_arr
tbpsg_export_to_workspace(toolboxstruc_arr);

% set values of confidence level in quantile regression 
alpha = [0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9];

% start confidence level cycle
for i=1:length(alpha)
 
% generate problem statement for Fidelity Magellan Fund   
    str1 = sprintf('minimize');
    str3 = sprintf('%.1f%s', alpha(i),'*pm_pen_Magellan(0.0, matrix_style_classification_fidelity_magellan)');
    str4 = sprintf('%.1f%s', (1-alpha(i)),'*pm_pen_g_Magellan(0.0, matrix_style_classification_fidelity_magellan)');
    str6 = sprintf('Solver: VAN, precision = 5');
    clear problem_statement;
    problem_statement = sprintf('%s\n', str1, str3, str4, str6);

%Uncomment the following line to open the problem in Toolbox Window:
%tbpsg_toolbox(problem_statement,toolboxstruc_arr);
    
% optimize problem  for Fidelity Magellan Fund     
    [solution_str, outargstruc_arr] = tbpsg_run(problem_statement, toolboxstruc_arr);

% extract optimal point for Fidelity Magellan Fund and pack it 
% to input array of structures   
     point_variables = tbpsg_optimal_point_vars(solution_str, outargstruc_arr);
     loc = tbpsg_optimal_point_data(solution_str, outargstruc_arr)';
     point_magellan(:,i)= loc;
     toolboxstruc_arr(3) = tbpsg_point_pack('point_1',loc,point_variables);
     
% extract optimal objective value  for Fidelity Magellan Fund
     objective_magellan(i) = tbpsg_objective(solution_str, outargstruc_arr); 
     
% calculate probabilities of exceeding of alpha-quantile for Fidelity Magellan Fund     
    probability_magellan(i) = tbpsg_function_value('pr_pen(0.0, matrix_style_classification_fidelity_magellan)', 'point_1', toolboxstruc_arr);

% generate problem statement for S&P 500 index   
    str1 = sprintf('minimize');
    str3 = sprintf('%.1f%s', alpha(i),'*pm_pen_SPX(0.0, matrix_style_classification_spx)');
    str4 = sprintf('%.1f%s', (1-alpha(i)),'*pm_pen_g_SPX(0.0, matrix_style_classification_spx)');
    str6 = sprintf('Solver: VAN, precision = 5');
    clear problem_statement;
    problem_statement = sprintf('%s\n', str1, str3, str4, str6);

%Uncomment the following line to open the problem in Toolbox Window:
%tbpsg_toolbox(problem_statement,toolboxstruc_arr);

% optimize problem for S&P 500 index     
    [solution_str, outargstruc_arr] = tbpsg_run(problem_statement, toolboxstruc_arr);

    clear optimal_point;
    clear point_variables;
    
% extract optimal point for S&P 500 index and pack it 
% to input array of structures   
     point_variables = tbpsg_optimal_point_vars(solution_str, outargstruc_arr);
     loc = tbpsg_optimal_point_data(solution_str, outargstruc_arr)';
     point_SP_500(:,i)= loc;
     toolboxstruc_arr(4) = tbpsg_point_pack('point_2',loc,point_variables);
     
% extract optimal objective value  for Fidelity Magellan Fund
     objective_SP_500(i) = tbpsg_objective(solution_str, outargstruc_arr); 
     
% calculate probabilities of exceeding of alpha-quantile for Fidelity Magellan Fund     
    probability_SP_500(i) = tbpsg_function_value('pr_pen(0.0, matrix_style_classification_spx)', 'point_2', toolboxstruc_arr);
end
    
fprintf('\nCPU time = %f\n\n', cputime-thistime);

% display and save tables with results of calculations
outfilename = 'CS_Style_Classification_Quantile_Regression_Output.txt';
fid = fopen(outfilename, 'w');
fprintf('\n\n Table 1. Fidelity Magellan Fund. Optimal Objective Values \n%12s', 'Alpha   Optimal Objective');
fprintf(fid, '\n\n Table 1. Fidelity Magellan Fund. Optimal Objective Values \n%12s', 'Alpha   Optimal Objective');
for i=1:1:length(alpha)
    fprintf('\n%5.1f %15.8f', alpha(i), objective_magellan(i));
    fprintf(fid,'\n%5.1f %15.8f', alpha(i), objective_magellan(i));
end

fprintf('\n\n Table 2. Fidelity Magellan Fund. Probability of Exceeding of alpha-quantile \n%12s', 'Alpha   Probability of Exceeding of alpha-quantile');
fprintf(fid,'\n\n Table 2. Fidelity Magellan Fund. Probability of Exceeding of alpha-quantile \n%12s', 'Alpha   Probability of Exceeding of alpha-quantile');
for i=1:1:length(alpha)
    fprintf('\n%5.1f %15.8f', alpha(i), probability_magellan(i));
    fprintf(fid,'\n%5.1f %15.8f', alpha(i), probability_magellan(i));
end

fprintf('\n\n Table 3. Fidelity Magellan Fund. Dependence of the Loading Factors upon the Confidence Level \n');
fprintf('alpha');
fprintf(fid,'\n\n Table 3. Fidelity Magellan Fund. Dependence of the Loading Factors upon the Confidence Level \n');
fprintf(fid,'alpha');
for j=1:1:length(point_variables)
    fprintf('%12s', point_variables{j});
    fprintf(fid,'%12s', point_variables{j});
end
for i=1:1:length(alpha)
    fprintf('\n%5.1f', alpha(i));
    fprintf(fid,'\n%5.1f', alpha(i));
    for j=1:1:length(point_variables)
        fprintf('%13.8f', point_magellan(j,i));
        fprintf(fid,'%13.8f', point_magellan(j,i));
    end
end

fprintf('\n\n Table 4. S&P 500 Index. Optimal Objective Values \n%12s', 'Alpha   Optimal Objective');
fprintf(fid,'\n\n Table 4. S&P 500 Index. Optimal Objective Values \n%12s', 'Alpha   Optimal Objective');
for i=1:1:length(alpha)
    fprintf('\n%5.1f %15.8f', alpha(i), objective_SP_500(i));
    fprintf(fid,'\n%5.1f %15.8f', alpha(i), objective_SP_500(i));
end

fprintf('\n\n Table 5. S&P 500 Index. Probability of Exceeding of alpha-quantile \n%12s', 'Alpha   Probability of Exceeding of alpha-quantile');
fprintf(fid,'\n\n Table 5. S&P 500 Index. Probability of Exceeding of alpha-quantile \n%12s', 'Alpha   Probability of Exceeding of alpha-quantile');
for i=1:1:length(alpha)
    fprintf('\n%5.1f %15.8f', alpha(i), probability_SP_500(i));
    fprintf(fid,'\n%5.1f %15.8f', alpha(i), probability_SP_500(i));
end

fprintf('\n\n Table 6. S&P 500 Index. Dependence of the Loading Factors upon the Confidence Level \n');
fprintf('alpha');
fprintf(fid,'\n\n Table 6. S&P 500 Index. Dependence of the Loading Factors upon the Confidence Level \n');
fprintf(fid,'alpha');
for j=1:1:length(point_variables)
    fprintf('%12s', point_variables{j});
    fprintf(fid,'%12s', point_variables{j});
end
for i=1:1:length(alpha)
    fprintf('\n%5.1f', alpha(i));
    fprintf(fid,'\n%5.1f', alpha(i));
    for j=1:1:length(point_variables)
        fprintf('%13.8f', point_SP_500(j,i));
        fprintf(fid,'%13.8f', point_SP_500(j,i));
    end
end
fprintf('\n\n');

warning('on', 'all');

% display Graph "Fidelity Magellan Fund. Objective vs. Confidence Levels"
figure
plot(alpha,objective_magellan);
grid on;
xlabel('Confidence Levels (alpha)');
ylabel('Objective Values');
title('Fidelity Magellan Fund. Objective vs. Confidence Levels ');

% display Graph "Fidelity Magellan Fund. Probability of Exceeding of
% alpha-quantile vs. Confidence Levels" 
figure
plot(alpha,probability_magellan);
grid on;
xlabel('Confidence Levels (alpha)');
ylabel('Probability of Exceeding of alpha-quantile');
title('Fidelity Magellan Fund. Probability of Exceeding of alpha-quantile vs. Confidence Levels ');

% display Graph "Fidelity Magellan Fund. Dependence of the loading factors
% upon the confidence level"
figure
plot(alpha,point_magellan);
grid on;
xlabel('Confidence Levels (alpha)');
ylabel('Values of Loading Factors');
title('Fidelity Magellan Fund. Dependence of the Loading Factors upon the Confidence Level ');
legend(upper(point_variables), 'Location', 'East');

% display Graph "S&P 500 Index. Objective vs. Confidence Levels"
figure
plot(alpha,objective_SP_500);
grid on;
xlabel('Confidence Levels (alpha)');
ylabel('Objective Values');
title('S&P 500 Index. Objective vs. Confidence Levels ');

% display Graph "FS&P 500 Index. Probability of Exceeding of
% alpha-quantile vs. Confidence Levels" 
figure
plot(alpha,probability_SP_500);
grid on;
xlabel('Confidence Levels (alpha)');
ylabel('Probability of Exceeding of alpha-quantile');
title('S&P 500 Index. Probability of Exceeding of alpha-quantile vs. Confidence Levels ');

% display Graph "S&P 500 Index. Dependence of the loading factors
% upon the confidence level"
figure
plot(alpha,point_SP_500);
grid on;
xlabel('Confidence Levels (alpha)');
ylabel('Values of Loading Factors');
title('S&P 500 Index. Dependence of the Loading Factors upon the Confidence Level ');
legend(upper(point_variables), 'Location', 'East');

%=======================================================================|
%American Optimal Decisions, Inc. Copyright                             |
%Copyright American Optimal Decisions, Inc. 2007-2016.                 |
%American Optimal Decisions (AOD) retains copyrights to this material.  |
%                                                                       |
%Permission to reproduce this document and to prepare derivative works  |
%from this document for internal use is granted, provided the copyright |
%and No Warranty  statements are included with all reproductions       |
%and derivative works.                                                  |
%                                                                       |
%For information regarding external or commercial use of copyrighted    |
%materials owned by AOD, contact AOD at support@aorda.com.              |
%=======================================================================|